@@ -21,7 +21,7 @@
 #include "Python.h"
 #include <math.h>
 
-#define PYO_VERSION "0.8.2"
+#define PYO_VERSION "0.8.6"
 
 #ifndef __MYFLT_DEF
 #define __MYFLT_DEF
@@ -207,6 +207,7 @@
 
 #ifdef USE_PORTMIDI
 extern PyTypeObject MidiListenerType;
+extern PyTypeObject MidiDispatcherType;
 #endif
 #ifdef USE_OSC
 extern PyTypeObject OscListenerType;
@@ -500,12 +501,17 @@ extern PyTypeObject ScopeType;
 extern PyTypeObject PeakAmpType;
 extern PyTypeObject MainParticleType;
 extern PyTypeObject ParticleType;
+extern PyTypeObject MainParticle2Type;
+extern PyTypeObject Particle2Type;
 extern PyTypeObject AtanTableType;
 extern PyTypeObject RawMidiType;
 extern PyTypeObject ResampleType;
 extern PyTypeObject ExprType;
 extern PyTypeObject PadSynthTableType;
 extern PyTypeObject LogiMapType;
+extern PyTypeObject SharedTableType;
+extern PyTypeObject TableFillType;
+extern PyTypeObject TableScanType;
 
 /* Constants */
 #define E M_E
@@ -692,7 +698,7 @@ extern PyTypeObject LogiMapType;
     else if (self->interp == 3) \
         self->interp_func_ptr = cosine; \
     else if (self->interp == 4) \
-        self->interp_func_ptr = cubic; \
+        self->interp_func_ptr = cubic;
 
 /* Set data */
 #define SET_TABLE_DATA \
@@ -1567,6 +1573,8 @@ extern PyTypeObject LogiMapType;
 #define PLAY \
     float del = 0; \
     float dur = 0; \
+    float globdel = 0; \
+    float globdur = 0; \
     int nearestBuf = 0; \
     int i; \
  \
@@ -1574,6 +1582,14 @@ extern PyTypeObject LogiMapType;
  \
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "|ff", kwlist, &dur, &del)) \
         return PyInt_FromLong(-1); \
+ \
+    globdel = PyFloat_AsDouble(PyObject_CallMethod(PyServer_get_server(), "getGlobalDel", NULL)); \
+    globdur = PyFloat_AsDouble(PyObject_CallMethod(PyServer_get_server(), "getGlobalDur", NULL)); \
+ \
+    if (globdel != 0) \
+        del = globdel; \
+    if (globdur != 0) \
+        dur = globdur; \
  \
     Stream_setStreamToDac(self->stream, 0); \
     if (del == 0) { \
@@ -1606,6 +1622,8 @@ extern PyTypeObject LogiMapType;
     int chnltmp = 0; \
     float del = 0; \
     float dur = 0; \
+    float globdel = 0; \
+    float globdur = 0; \
     int nearestBuf = 0; \
     int i; \
 \
@@ -1613,6 +1631,14 @@ extern PyTypeObject LogiMapType;
  \
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "|iff", kwlist, &chnltmp, &dur, &del)) \
         return PyInt_FromLong(-1); \
+ \
+    globdel = PyFloat_AsDouble(PyObject_CallMethod(PyServer_get_server(), "getGlobalDel", NULL)); \
+    globdur = PyFloat_AsDouble(PyObject_CallMethod(PyServer_get_server(), "getGlobalDur", NULL)); \
+ \
+    if (globdel != 0) \
+        del = globdel; \
+    if (globdur != 0) \
+        dur = globdur; \
  \
     Stream_setStreamChnl(self->stream, chnltmp % self->nchnls); \
     Stream_setStreamToDac(self->stream, 1); \
