@@ -76,10 +76,15 @@ typedef struct {
     int jackautoout; /* jack port auto-connection (on by default) */
     PyObject *jackAutoConnectInputPorts; /* list of lists of jack auto-connection ports to pyo inputs */
     PyObject *jackAutoConnectOutputPorts; /* list of lists of jack auto-connection ports from pyo outputs */
+    PyObject *jackInputPortNames; /* string or list of strings (input port short names for jack server */
+    PyObject *jackOutputPortNames; /* string or list of strings (output port short names for jack server */
+    int isJackTransportSlave;
+    int jack_transport_state; /* 0 = stopped, 1 = started */
     PyoMidiEvent midiEvents[200];
     int midiin_count;
     int midiout_count;
     int midi_count;
+    long midi_time_offset;
     double samplingRate;
     int nchnls;
     int ichnls;
@@ -144,6 +149,10 @@ typedef struct {
     /* custom callback */
     PyObject *CALLBACK;
 
+    /* Globals dur and del times. */
+    float globalDur;
+    float globalDel;
+
     /* Properties */
     int verbosity; /* a sum of values to display different levels: 1 = error */
                    /* 2 = message, 4 = warning , 8 = debug. Default 7.*/
@@ -156,6 +165,8 @@ extern PyObject * Server_removeStream(Server *self, int sid);
 extern MYFLT * Server_getInputBuffer(Server *self);
 extern PyoMidiEvent * Server_getMidiEventBuffer(Server *self);
 extern int Server_getMidiEventCount(Server *self);
+extern long Server_getMidiTimeOffset(Server *self);
+extern unsigned long Server_getElapsedTime(Server *self);
 extern int Server_getCurrentResamplingFactor(Server *self);
 extern int Server_getLastResamplingFactor(Server *self);
 extern int Server_generateSeed(Server *self, int oid);
@@ -166,8 +177,10 @@ void Server_error(Server *self, char * format, ...);
 void Server_message(Server *self, char * format, ...);
 void Server_warning(Server *self, char * format, ...);
 void Server_debug(Server *self, char * format, ...);
-PyObject * Server_shut_down(Server *self);
-PyObject *Server_stop(Server *self);
+PyObject * Server_shutdown(Server *self);
+PyObject * Server_stop(Server *self);
+PyObject * Server_start(Server *self);
+PyObject * Server_boot(Server *self, PyObject *arg);
 void Server_process_gui(Server *server);
 void Server_process_time(Server *server);
 int Server_start_rec_internal(Server *self, char *filename);
