@@ -1,3 +1,8 @@
+# encoding: utf-8
+"""
+This module defines GUI widgets built with tkinter.
+
+"""
 from __future__ import division
 from __future__ import print_function
 from __future__ import absolute_import
@@ -20,11 +25,12 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-import math, sys, os
+import math
+import sys
 if sys.version_info[0] < 3:
-    from Tkinter import *
+    import Tkinter as tk
 else:
-    from tkinter import *
+    import tkinter as tk
 
 # constants for platform displays with Tk
 if sys.platform.startswith('linux'):
@@ -40,32 +46,47 @@ else:
 ######################################################################
 ### Multisliders
 ######################################################################
-class MultiSlider(Frame):
+class MultiSlider(tk.Frame):
+    """
+    Draw multiple sliders on a canvas.
 
+    :Args:
+        master: tk.Frame
+            Parent frame.
+        init: list of floats
+            Initial values.
+        key: str
+            Name of the attribute to control.
+        command: callable
+            Function called with values as argument.
+
+    """
     def __init__(self, master, init, key, command):
-        Frame.__init__(self, master, bd=0, relief=FLAT)
+        tk.Frame.__init__(self, master, bd=0, relief=tk.FLAT)
         self._values = init
         self._nchnls = len(init)
         self._key = key
         self._command = command
         self._lines = []
         self._height = 16
-        self.canvas = Canvas(self, height=self._height*self._nchnls+1,
-                            width=225, relief=FLAT, bd=0, bg="#BCBCAA")
+        self.canvas = tk.Canvas(self, height=self._height*self._nchnls+1,
+                                width=225, relief=tk.FLAT, bd=0, bg="#BCBCAA")
         w = self.canvas.winfo_width()
         for i in range(self._nchnls):
             x = int(self._values[i] * w)
             y = self._height * i + Y_OFFSET
-            self._lines.append(self.canvas.create_rectangle(0, y, x,
-                                y+self._height-1, width=0, fill="#121212"))
+            rect = self.canvas.create_rectangle(0, y, x, y+self._height-1,
+                                                width=0, fill="#121212")
+            self._lines.append(rect)
         self.canvas.bind("<Button-1>", self.clicked)
         self.canvas.bind("<Motion>", self.move)
         self.canvas.bind("<Configure>", self.size)
-        self.canvas.grid(sticky=E+W)
+        self.canvas.grid(sticky=tk.E + tk.W)
         self.columnconfigure(0, weight=1)
         self.grid()
 
     def size(self, event):
+        "Method called when resizing the window."
         w = self.canvas.winfo_width()
         for i in range(len(self._lines)):
             y = self._height * i + Y_OFFSET
@@ -73,15 +94,18 @@ class MultiSlider(Frame):
             self.canvas.coords(self._lines[i], 0, y, x, y+self._height-1)
 
     def clicked(self, event):
+        "Method called when clicking on the window."
         self.update(event)
 
     def move(self, event):
+        "Method called on left mouse moion."
         if event.state == 0x0100:
             slide = (event.y - Y_OFFSET) // self._height
             if 0 <= slide < len(self._lines):
                 self.update(event)
 
     def update(self, event):
+        "Update and send current values."
         w = self.canvas.winfo_width()
         slide = (event.y - Y_OFFSET) // self._height
         val = event.x / float(w)
@@ -93,8 +117,8 @@ class MultiSlider(Frame):
 ######################################################################
 ### Control window for PyoObject
 ######################################################################
-class Command:
-
+class Command(object):
+    "Command event used to control a slider value."
     def __init__(self, func, key):
         self.func = func
         self.key = key
@@ -102,10 +126,21 @@ class Command:
     def __call__(self, value):
         self.func(self.key, value)
 
-class PyoObjectControl(Frame):
+class PyoObjectControl(tk.Frame):
+    """
+    Create a slider/multislider control window.
 
+    :Args:
+        master: tk.Frame
+            Parent frame.
+        obj: PyoObjectBase
+            The object to control attributes.
+        map_list: list of SLMap
+            The list of attribute definitions.
+
+    """
     def __init__(self, master=None, obj=None, map_list=None):
-        Frame.__init__(self, master, bd=1, relief=GROOVE)
+        tk.Frame.__init__(self, master, bd=1, relief=tk.GROOVE)
         from .controls import SigTo
         self.bind('<Destroy>', self._destroy)
         self._obj = obj
@@ -124,25 +159,32 @@ class PyoObjectControl(Frame):
             else:
                 self._maps[key] = m
                 # label (param name)
-                label = Label(self, height=1, width=10, highlightthickness=0, text=key)
+                label = tk.Label(self, height=1, width=10, highlightthickness=0,
+                                 text=key)
                 label.grid(row=i, column=0)
                 # create and pack slider
-                if type(init) != list:
-                    self._sliders.append(Scale(self, command=Command(self.setval, key),
-                                  orient=HORIZONTAL, relief=GROOVE, from_=0., to=1., showvalue=False,
-                                  resolution=.0001, bd=1, length=225, troughcolor="#BCBCAA", width=12))
+                if not isinstance(init, list):
+                    slider = tk.Scale(self, command=Command(self.setval, key),
+                                      orient=tk.HORIZONTAL, relief=tk.GROOVE,
+                                      from_=0., to=1., showvalue=False,
+                                      resolution=.0001, bd=1, length=225,
+                                      troughcolor="#BCBCAA", width=12)
+                    self._sliders.append(slider)
                     self._sliders[-1].set(m.set(init))
                     disp_height = 1
                 else:
-                    self._sliders.append(MultiSlider(self, [m.set(x) for x in init], key, self.setval))
+                    self._sliders.append(MultiSlider(self,
+                                                     [m.set(x) for x in init],
+                                                     key, self.setval))
                     disp_height = len(init)
-                self._sliders[-1].grid(row=i, column=1, sticky=E+W)
+                self._sliders[-1].grid(row=i, column=1, sticky=tk.E + tk.W)
                 # display of numeric values
-                textvar = StringVar(self)
-                display = Label(self, height=disp_height, width=10, highlightthickness=0, textvariable=textvar)
+                textvar = tk.StringVar(self)
+                display = tk.Label(self, height=disp_height, width=10,
+                                   highlightthickness=0, textvariable=textvar)
                 display.grid(row=i, column=2)
                 self._displays[key] = textvar
-                if type(init) != list:
+                if not isinstance(init, list):
                     self._displays[key].set("%.4f" % init)
                 else:
                     self._displays[key].set("\n".join(["%.4f" % i for i in init]))
@@ -159,7 +201,7 @@ class PyoObjectControl(Frame):
         top.rowconfigure(0, weight=1)
         top.columnconfigure(0, weight=1)
         self.columnconfigure(1, weight=1)
-        self.grid(ipadx=5, ipady=5, sticky=E+W)
+        self.grid(ipadx=5, ipady=5, sticky=tk.E + tk.W)
 
     def _destroy(self, event):
         for m in self._map_list:
@@ -169,7 +211,8 @@ class PyoObjectControl(Frame):
                 del self._sigs[key]
 
     def setval(self, key, x):
-        if type(x) != list:
+        "Set a new value 'x' to attribute 'key'."
+        if not isinstance(x, list):
             value = self._maps[key].get(float(x))
             self._displays[key].set("%.4f" % value)
         else:
@@ -182,15 +225,18 @@ class PyoObjectControl(Frame):
 ######################################################################
 ### View window for PyoTableObject
 ######################################################################
-class ViewTable(Frame):
-
+class ViewTable(tk.Frame):
+    "PyoTableObject display."
     def __init__(self, master=None, samples=None):
-        Frame.__init__(self, master, bd=1, relief=GROOVE)
+        tk.Frame.__init__(self, master, bd=1, relief=tk.GROOVE)
         self.width = 500
         self.height = 200
         self.half_height = self.height // 2
-        self.canvas = Canvas(self, height=self.height, width=self.width, relief=SUNKEN, bd=1, bg="#EFEFEF")
-        self.canvas.create_line(0, self.half_height+Y_OFFSET, self.width, self.half_height+Y_OFFSET, fill='grey', dash=(4,2))
+        self.canvas = tk.Canvas(self, height=self.height, width=self.width,
+                                relief=tk.SUNKEN, bd=1, bg="#EFEFEF")
+        self.canvas.create_line(0, self.half_height+Y_OFFSET, self.width,
+                                self.half_height+Y_OFFSET, fill='grey',
+                                dash=(4, 2))
         self.canvas.create_line(*samples)
         self.canvas.grid()
         self.grid(ipadx=10, ipady=10)
@@ -198,13 +244,14 @@ class ViewTable(Frame):
 ######################################################################
 ## View window for PyoMatrixObject
 #####################################################################
-class ViewMatrix(Frame):
-
+class ViewMatrix(tk.Frame):
+    "PyoMatrixObject display."
     def __init__(self, master=None, samples=None, size=None):
-        Frame.__init__(self, master, bd=1, relief=GROOVE)
+        tk.Frame.__init__(self, master, bd=1, relief=tk.GROOVE)
         self.width = size[0]
         self.height = size[1]
-        self.canvas = Canvas(self, width=self.width, height=self.height, relief=SUNKEN, bd=1, bg="#EFEFEF")
+        self.canvas = tk.Canvas(self, width=self.width, height=self.height,
+                                relief=tk.SUNKEN, bd=1, bg="#EFEFEF")
         for i in range(self.width*self.height):
             x = i % self.width
             y = i // self.width
@@ -224,11 +271,12 @@ class ViewMatrix(Frame):
 ######################################################################
 ### Server Object User Interface (Tk)
 ######################################################################
-class ServerGUI(Frame):
-
-    def __init__(self, master=None, nchnls=2, startf=None, stopf=None, recstartf=None,
-                recstopf=None, ampf=None, started=0, locals=None, shutdown=None, meter=True, timer=True, amp=1.):
-        Frame.__init__(self, master, padx=10, pady=10, bd=2, relief=GROOVE)
+class ServerGUI(tk.Frame):
+    "Server's graphical interface."
+    def __init__(self, master=None, nchnls=2, startf=None, stopf=None,
+                 recstartf=None, recstopf=None, ampf=None, started=0,
+                 locals=None, shutdown=None, meter=True, timer=True, amp=1.):
+        tk.Frame.__init__(self, master, padx=10, pady=10, bd=2, relief=tk.GROOVE)
         self.shutdown = shutdown
         self.locals = locals
         self.meter = meter
@@ -254,67 +302,84 @@ class ServerGUI(Frame):
             self.start(True)
 
     def createWidgets(self):
+        "Interface builder."
         row = 0
-        self.startStringVar = StringVar(self)
+        self.startStringVar = tk.StringVar(self)
         self.startStringVar.set('Start')
-        self.startButton = Button(self, textvariable=self.startStringVar, command=self.start)
+        self.startButton = tk.Button(self, textvariable=self.startStringVar,
+                                     command=self.start)
         self.startButton.grid(ipadx=5)
 
-        self.recStringVar = StringVar(self)
+        self.recStringVar = tk.StringVar(self)
         self.recStringVar.set('Rec Start')
-        self.recButton = Button(self, textvariable=self.recStringVar, command=self.record)
+        self.recButton = tk.Button(self, textvariable=self.recStringVar,
+                                   command=self.record)
         self.recButton.grid(ipadx=5, row=row, column=1)
 
-        self.quitButton = Button(self, text='Quit', command=self.on_quit)
+        self.quitButton = tk.Button(self, text='Quit', command=self.on_quit)
         self.quitButton.grid(ipadx=5, row=row, column=2)
         row += 1
 
-        self.ampScale = Scale(self, command=self.setAmp, digits=4, label='Amplitude (dB)',
-                              orient=HORIZONTAL, relief=GROOVE, from_=-60.0, to=18.0,
-                              resolution=.01, bd=1, length=250, troughcolor="#BCBCAA", width=10)
+        self.ampScale = tk.Scale(self, command=self.setAmp, digits=4,
+                                 label='Amplitude (dB)', orient=tk.HORIZONTAL,
+                                 relief=tk.GROOVE, from_=-60.0, to=18.0,
+                                 resolution=.01, bd=1, length=250,
+                                 troughcolor="#BCBCAA", width=10)
         self.ampScale.set(20.0 * math.log10(self.amp))
         self.ampScale.grid(ipadx=5, ipady=5, row=row, column=0, columnspan=3)
         row += 1
 
         if self.meter:
-            self.vumeter = Canvas(self, height=5*self.nchnls+1, width=250, relief=FLAT, bd=0, bg="#323232")
+            self.vumeter = tk.Canvas(self, height=5*self.nchnls+1, width=250,
+                                     relief=tk.FLAT, bd=0, bg="#323232")
             self.green = []
             self.yellow = []
             self.red = []
             for i in range(self.nchnls):
                 y = 5 * (i + 1) + 1 - VM_OFFSET
-                self.green.append(self.vumeter.create_line(0, y, 1, y, width=4, fill='green', dash=(9,1), dashoff=6+VM_OFFSET))
-                self.yellow.append(self.vumeter.create_line(self.B1, y, self.B1, y, width=4, fill='yellow', dash=(9,1), dashoff=9))
-                self.red.append(self.vumeter.create_line(self.B2, y, self.B2, y, width=4, fill='red', dash=(9,1), dashoff=0))
+                self.green.append(self.vumeter.create_line(0, y, 1, y, width=4,
+                                                           fill='green',
+                                                           dash=(9, 1),
+                                                           dashoff=6+VM_OFFSET))
+                self.yellow.append(self.vumeter.create_line(self.B1, y, self.B1,
+                                                            y, width=4,
+                                                            fill='yellow',
+                                                            dash=(9, 1),
+                                                            dashoff=9))
+                self.red.append(self.vumeter.create_line(self.B2, y, self.B2, y,
+                                                         width=4, fill='red',
+                                                         dash=(9, 1), dashoff=0))
             self.vumeter.grid(ipadx=5, row=row, column=0, columnspan=3)
             row += 1
 
         if self.timer:
-            self.timer_label = Label(self, text='Elapsed time (h:m:s:ms)')
+            self.timer_label = tk.Label(self, text='Elapsed time (h:m:s:ms)')
             self.timer_label.grid(ipadx=0, row=row, column=0, columnspan=3)
             row += 1
-            self.timer_strvar = StringVar(self, " 00 : 00 : 00 : 000")
-            self.timetext = Label(self, textvariable=self.timer_strvar)
+            self.timer_strvar = tk.StringVar(self, " 00 : 00 : 00 : 000")
+            self.timetext = tk.Label(self, textvariable=self.timer_strvar)
             self.timetext.grid(ipadx=5, row=row, column=0, columnspan=3)
             row += 1
 
         if self.locals is not None:
-            self.interp_label = Label(self, text='Interpreter')
+            self.interp_label = tk.Label(self, text='Interpreter')
             self.interp_label.grid(ipadx=0, row=row, column=0, columnspan=3)
             row += 1
-            self.text = Text(self, height=1, width=33, bd=1, relief=RIDGE, highlightthickness=0,
-                            spacing1=2, spacing3=2)
+            self.text = tk.Text(self, height=1, width=33, bd=1, relief=tk.RIDGE,
+                                highlightthickness=0, spacing1=2, spacing3=2)
             self.text.grid(ipadx=5, row=row, column=0, columnspan=3)
             self.text.bind("<Return>", self.getText)
             self.text.bind("<Up>", self.getPrev)
             self.text.bind("<Down>", self.getNext)
 
     def on_quit(self):
+        "Clean up on quit."
         self.shutdown()
         self.quit()
 
     def getPrev(self, event):
-        self.text.delete("1.0", END)
+        "Get previous command in history."
+        self.text.delete("1.0", tk.END)
         self._histo_count -= 1
         if self._histo_count < 0:
             self._histo_count = 0
@@ -322,10 +387,13 @@ class ServerGUI(Frame):
         return "break"
 
     def setTime(self, *args):
-        self.timer_strvar.set(" %02d : %02d : %02d : %03d" % (args[0], args[1], args[2], args[3]))
+        "Display the current server time."
+        txt = " %02d : %02d : %02d : %03d" % (args[0], args[1], args[2], args[3])
+        self.timer_strvar.set(txt)
 
     def getNext(self, event):
-        self.text.delete("1.0", END)
+        "Get next command in history."
+        self.text.delete("1.0", tk.END)
         self._histo_count += 1
         if self._histo_count >= len(self._history):
             self._histo_count = len(self._history)
@@ -334,28 +402,31 @@ class ServerGUI(Frame):
         return "break"
 
     def getText(self, event):
-        source = self.text.get("1.0", END)
-        self.text.delete("1.0", END)
+        "Retrieve text from the interpreter widget."
+        source = self.text.get("1.0", tk.END)
+        self.text.delete("1.0", tk.END)
         exec(source, self.locals)
         self._history.append(source)
         self._histo_count = len(self._history)
         return "break"
 
     def start(self, justSet=False):
-        if self._started == False:
+        "Start/stop the server."
+        if self._started is False:
             if not justSet:
                 self.startf()
             self._started = True
             self.startStringVar.set('Stop')
-            self.quitButton.configure(state = DISABLED)
+            self.quitButton.configure(state=tk.DISABLED)
         else:
             self.stopf()
             self._started = False
             self.startStringVar.set('Start')
-            self.quitButton.configure(state = NORMAL)
+            self.quitButton.configure(state=tk.NORMAL)
 
     def record(self):
-        if self._recstarted == False:
+        "Record sound on disk."
+        if self._recstarted is False:
             self.recstartf()
             self._recstarted = True
             self.recStringVar.set('Rec Stop')
@@ -365,9 +436,11 @@ class ServerGUI(Frame):
             self.recStringVar.set('Rec Start')
 
     def setAmp(self, value):
+        "Update server's global amplitude value."
         self.ampf(math.pow(10.0, float(value) * 0.05))
 
     def setRms(self, *args):
+        "Update the vumeter."
         for i in range(self.nchnls):
             y = 5 * (i + 1) + 1 - VM_OFFSET
             db = math.log10(args[i]+0.00001) * 0.2 + 1.
@@ -384,3 +457,13 @@ class ServerGUI(Frame):
                 self.vumeter.coords(self.green[i], 0, y, self.B1, y)
                 self.vumeter.coords(self.yellow[i], self.B1, y, self.B2, y)
                 self.vumeter.coords(self.red[i], self.B2, y, amp, y)
+
+    def setStartButtonState(self, state):
+        if state:
+            self._started = True
+            self.startStringVar.set('Stop')
+            self.quitButton.configure(state=tk.DISABLED)
+        else:
+            self._started = False
+            self.startStringVar.set('Start')
+            self.quitButton.configure(state=tk.NORMAL)
