@@ -80,6 +80,7 @@ class Midictl(PyoObject):
         self._channel = channel
         ctlnumber, minscale, maxscale, init, channel, mul, add, lmax = convertArgsToLists(ctlnumber, minscale, maxscale, init, channel, mul, add)
         self._base_objs = [Midictl_base(wrap(ctlnumber,i), wrap(minscale,i), wrap(maxscale,i), wrap(init,i), wrap(channel,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -160,17 +161,10 @@ class Midictl(PyoObject):
 
     def setInterpolation(self, x):
         """
-        Activate/Deactivate interpolation. Off by default.
-
-        :Args:
-
-            x: boolean
-                True activates the interpolation, False deactivates it.
+        Deprecated method. If needed, use Port or SigTo to interpolate between values.
 
         """
-        pyoArgsAssert(self, "b", x)
-        x, lmax = convertArgsToLists(x)
-        [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+        print("setInterpolation() is deprecated. If needed, use Port or SigTo to interpolate between values.")
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, 127, 'lin', 'ctlnumber', self._ctlnumber, res="int", dataOnly=True),
@@ -255,6 +249,7 @@ class CtlScan(PyoObject):
         self._function = WeakMethod(function)
         self._toprint = toprint
         self._base_objs = [CtlScan_base(self._function, self._toprint)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -366,6 +361,7 @@ class CtlScan2(PyoObject):
         self._function = WeakMethod(function)
         self._toprint = toprint
         self._base_objs = [CtlScan2_base(self._function, self._toprint)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -514,6 +510,7 @@ class Notein(PyoObject):
             self._base_objs.append(Notein_base(self._base_handler, i, 1, wrap(mul,i), wrap(add,i)))
             self._trig_objs.append(NoteinTrig_base(self._base_handler, i, 0, 1, 0))
             self._trig_objs.append(NoteinTrig_base(self._base_handler, i, 1, 1, 0))
+        self.play()
 
     def __getitem__(self, str):
         if str == 'pitch':
@@ -740,6 +737,7 @@ class Bendin(PyoObject):
         self._channel = channel
         brange, scale, channel, mul, add, lmax = convertArgsToLists(brange, scale, channel, mul, add)
         self._base_objs = [Bendin_base(wrap(brange,i), wrap(scale,i), wrap(channel,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -791,17 +789,10 @@ class Bendin(PyoObject):
 
     def setInterpolation(self, x):
         """
-        Activate/Deactivate interpolation. Off by default.
-
-        :Args:
-
-            x: boolean
-                True activates the interpolation, False deactivates it.
+        Deprecated method. If needed, use Port or SigTo to interpolate between values.
 
         """
-        pyoArgsAssert(self, "b", x)
-        x, lmax = convertArgsToLists(x)
-        [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+        print("setInterpolation() is deprecated. If needed, use Port or SigTo to interpolate between values.")
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0.0, 12.0, 'lin', 'brange', self._brange, dataOnly=True),
@@ -874,6 +865,7 @@ class Touchin(PyoObject):
         self._channel = channel
         minscale, maxscale, init, channel, mul, add, lmax = convertArgsToLists(minscale, maxscale, init, channel, mul, add)
         self._base_objs = [Touchin_base(wrap(minscale,i), wrap(maxscale,i), wrap(init,i), wrap(channel,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -925,17 +917,10 @@ class Touchin(PyoObject):
 
     def setInterpolation(self, x):
         """
-        Activate/Deactivate interpolation. Off by default.
-
-        :Args:
-
-            x: boolean
-                True activates the interpolation, False deactivates it.
+        Deprecated method. If needed, use Port or SigTo to interpolate between values.
 
         """
-        pyoArgsAssert(self, "b", x)
-        x, lmax = convertArgsToLists(x)
-        [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+        print("setInterpolation() is deprecated. If needed, use Port or SigTo to interpolate between values.")
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(-1.0, 0.0, 'lin', 'minscale', self._minscale, dataOnly=True),
@@ -1002,6 +987,7 @@ class Programin(PyoObject):
         self._channel = channel
         channel, mul, add, lmax = convertArgsToLists(channel, mul, add)
         self._base_objs = [Programin_base(wrap(channel,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -1089,6 +1075,7 @@ class MidiAdsr(PyoObject):
         self._in_fader = InputFader(input)
         in_fader, attack, decay, sustain, release, mul, add, lmax = convertArgsToLists(self._in_fader, attack, decay, sustain, release, mul, add)
         self._base_objs = [MidiAdsr_base(wrap(in_fader,i), wrap(attack,i), wrap(decay,i), wrap(sustain,i), wrap(release,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -1293,6 +1280,7 @@ class MidiDelAdsr(PyoObject):
         self._in_fader = InputFader(input)
         in_fader, delay, attack, decay, sustain, release, mul, add, lmax = convertArgsToLists(self._in_fader, delay, attack, decay, sustain, release, mul, add)
         self._base_objs = [MidiDelAdsr_base(wrap(in_fader,i), wrap(delay,i), wrap(attack,i), wrap(decay,i), wrap(sustain,i), wrap(release,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
@@ -1497,6 +1485,7 @@ class RawMidi(PyoObject):
         PyoObject.__init__(self)
         self._function = WeakMethod(function)
         self._base_objs = [RawMidi_base(self._function)]
+        self.play()
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
