@@ -71,6 +71,7 @@ class Pan(PyoObject):
         for i in range(lmax):
             for j in range(outs):
                 self._base_objs.append(Pan_base(wrap(self._base_players,i), j, wrap(mul,i), wrap(add,i)))
+        self.play()
 
     def setInput(self, x, fadetime=0.05):
         """
@@ -182,6 +183,7 @@ class SPan(PyoObject):
         for i in range(lmax):
             for j in range(outs):
                 self._base_objs.append(SPan_base(wrap(self._base_players,i), j, wrap(mul,i), wrap(add,i)))
+        self.play()
 
     def setInput(self, x, fadetime=0.05):
         """
@@ -278,6 +280,7 @@ class Switch(PyoObject):
         for j in range(outs):
             for i in range(lmax):
                 self._base_objs.append(Switch_base(wrap(self._base_players,i), j, wrap(mul,i), wrap(add,i)))
+        self.play()
 
     def setInput(self, x, fadetime=0.05):
         """
@@ -359,6 +362,7 @@ class Selector(PyoObject):
         PyoObject.__init__(self, mul, add)
         self._inputs = inputs
         self._voice = voice
+        self._mode = 0
         voice, mul, add, self._lmax = convertArgsToLists(voice, mul, add)
         self._length = 1
         for obj in self._inputs:
@@ -376,6 +380,7 @@ class Selector(PyoObject):
                     except:
                         choice.append(obj)
                 self._base_objs.append(Selector_base(choice, wrap(voice,i), wrap(mul,i), wrap(add,i)))
+        self.play()
 
     def setInputs(self, x):
         """
@@ -415,6 +420,24 @@ class Selector(PyoObject):
         for i, obj in enumerate(self._base_objs):
             obj.setVoice(wrap(x, i // self._length))
 
+    def setMode(self, x):
+        """
+        Change the algorithm used to interpolate between inputs.
+
+        if inputs are phase correlated you should use a linear fade. 
+
+        :Args:
+
+            x: int {0, 1}
+                If 0 (the default) the equal power law is used to
+                interpolate bewtween sources. If 1, linear fade is
+                used instead.
+
+        """
+        pyoArgsAssert(self, "i", x)
+        self._mode = x
+        [obj.setMode(x) for obj in self._base_objs]
+
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, len(self._inputs)-1, "lin", "voice", self._voice), SLMapMul(self._mul)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
@@ -494,6 +517,7 @@ class VoiceManager(PyoObject):
         else:
             t_streams = None
         self._base_objs = [VoiceManager_base(wrap(in_fader,i), t_streams, wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+        self.play()
 
     def setInput(self, x, fadetime=0.05):
         """
@@ -604,6 +628,7 @@ class Mixer(PyoObject):
         time, mul, add, lmax = convertArgsToLists(time, mul, add)
         self._base_players = [Mixer_base(outs, wrap(time,i)) for i in range(chnls)]
         self._base_objs = [MixerVoice_base(self._base_players[j], i, wrap(mul,i), wrap(add,i)) for i in range(outs) for j in range(chnls)]
+        self.play()
 
     def __getitem__(self, x):
         if type(x) == slice:
