@@ -20,7 +20,7 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-import os, time
+import os, sys, time
 from ._core import *
 from ._widgets import createServerGUI
 
@@ -82,6 +82,9 @@ class Server(object):
             Name of jack client. Defaults to 'pyo'
         ichnls: int, optional
             Number of input channels if different of output channels. If None (default), ichnls = nchnls.
+        winhost: string, optional
+            Under Windows, pyo's Server will try to use the default devices of the given host.
+            This behaviour can be changed with the SetXXXDevice methods.
 
     .. note::
 
@@ -102,7 +105,8 @@ class Server(object):
         - setDuplex(x): Set the duplex mode used by the server.
         - setVerbosity(x): Set the server's verbosity.
         - reinit(sr, nchnls, buffersize, duplex, audio, jackname): Reinit the server's settings.
-        - deactivateMidi(): Deactivate Midi callback. 
+        - deactivateMidi(): Deactivate Midi callback.
+        - setIsJackTransportSlave(x): Set if pyo's server is slave to jack transport or not.
 
     >>> # For an 8 channels server in duplex mode with
     >>> # a sampling rate of 48000 Hz and buffer size of 512
@@ -111,7 +115,7 @@ class Server(object):
 
     """
     def __init__(self, sr=44100, nchnls=2, buffersize=256, duplex=1,
-                 audio='portaudio', jackname='pyo', ichnls=None):
+                 audio='portaudio', jackname='pyo', ichnls=None, winhost="wasapi"):
         if "PYO_SERVER_AUDIO" in os.environ and "offline" not in audio and "embedded" not in audio:
             audio = os.environ["PYO_SERVER_AUDIO"]
         self._time = time
@@ -120,6 +124,7 @@ class Server(object):
             self._ichnls = nchnls
         else:
             self._ichnls = ichnls
+        self._winhost = winhost
         self._amp = 1.
         self._verbosity = 7
         self._startoffset = 0
@@ -127,9 +132,17 @@ class Server(object):
         self._filename = None
         self._fileformat = 0
         self._sampletype = 0
+        self._globalseed = 0
+        self._resampling = 1
+        self._isJackTransportSlave = False
         self._server = Server_base(sr, nchnls, buffersize, duplex, audio, jackname, self._ichnls)
         self._server._setDefaultRecPath(os.path.join(os.path.expanduser("~"), "pyo_rec.wav"))
 
+        if sys.platform.startswith("win"):
+            host_default_in, host_default_out = pa_get_default_devices_from_host(winhost)
+            self._server.setInputDevice(host_default_in)
+            self._server.setOutputDevice(host_default_out)
+
     def __del__(self):
         self.setTime = None
         self.setRms = None
@@ -141,7 +154,7 @@ class Server(object):
             self._time.sleep(.25)
 
     def reinit(self, sr=44100, nchnls=2, buffersize=256, duplex=1,
-               audio='portaudio', jackname='pyo', ichnls=None):
+               audio='portaudio', jackname='pyo', ichnls=None, winhost="wasapi"):
         """
         Reinit the server'settings. Useful to alternate between real-time and offline server.
 
@@ -156,6 +169,7 @@ class Server(object):
             self._ichnls = nchnls
         else:
             self._ichnls = ichnls
+        self._winhost = winhost
         self._amp = 1.
         self._verbosity = 7
         self._startoffset = 0
@@ -165,8 +179,14 @@ class Server(object):
         self._sampletype = 0
         self._globalseed = 0
         self._resampling = 1
+        self._isJackTransportSlave = False
         self._server.__init__(sr, nchnls, buffersize, duplex, audio, jackname, self._ichnls)
 
+        if sys.platform.startswith("win"):
+            host_default_in, host_default_out = pa_get_default_devices_from_host(winhost)
+            self._server.setInputDevice(host_default_in)
+            self._server.setOutputDevice(host_default_out)
+
     def setCallback(self, callback):
         """
         Register a custom process callback.
@@ -180,7 +200,7 @@ class Server(object):
         if callable(callback):
             self._server.setCallback(callback)
 
-    def gui(self, locals=None, meter=True, timer=True, exit=True):
+    def gui(self, locals=None, meter=True, timer=True, exit=True, title=None):
         """
         Show the server's user interface.
 
@@ -199,12 +219,15 @@ class Server(object):
                 If True, the python interpreter will exit when the 'Quit' button is pressed,
                 Otherwise, the GUI will be closed leaving the interpreter alive.
                 Defaults to True.
+            title: str, optional
+                Alternate title for the server window. If None (default), generic
+                title, "Pyo Server" is used.
 
         """
         self._gui_frame, win = createServerGUI(self._nchnls, self.start, self.stop,
                                                self.recstart, self.recstop, self.setAmp,
                                                self.getIsStarted(), locals, self.shutdown,
-                                               meter, timer, self._amp, exit)
+                                               meter, timer, self._amp, exit, title)
         if meter:
             self._server.setAmpCallable(self._gui_frame)
         if timer:
@@ -460,6 +483,36 @@ class Server(object):
         self._verbosity = x
         self._server.setVerbosity(x)
 
+    def setGlobalDur(self, x):
+        """
+        Set the global object duration (time to wait before stopping the object).
+
+        This value, if not 0, will override the `dur` argument of object's
+        play() and out() methods.
+
+        :Args:
+
+            x: float
+                New global duration.
+
+        """
+        self._server.setGlobalDur(x)
+
+    def setGlobalDel(self, x):
+        """
+        Set the global object delay time (time to wait before activating the object).
+
+        This value, if not 0, will override the `del` argument of object's
+        play() and out() methods.
+
+        :Args:
+
+            x: float
+                New global delay time.
+
+        """
+        self._server.setGlobalDel(x)
+
     def setJackAuto(self, xin=True, xout=True):
         """
         Tells the server to auto-connect (or not) Jack ports to System ports.
@@ -506,6 +559,56 @@ class Server(object):
         ports, lmax = convertArgsToLists(ports)
         self._server.setJackAutoConnectOutputPorts(ports)
 
+    def setJackInputPortNames(self, name):
+        """
+        Change the short name of pyo's input ports for the jack server.
+
+        This method must be called after the server is booted.
+
+        :Args:
+
+            name: string or list of strings
+                New name of input ports for the jack server. If `name` is a string,
+                '_xxx' (where xxx is the channel number) will be added to it for
+                each input channel. If `name` is a list of strings, They will be
+                used as is and there must be one for each input channel.
+
+        """
+        self._server.setJackInputPortNames(name)
+
+    def setJackOutputPortNames(self, name):
+        """
+        Change the short name of pyo's output ports for the jack server.
+
+        This method must be called after the server is booted.
+
+        :Args:
+
+            name: string or list of strings
+                New name of output ports for the jack server. If `name` is a string,
+                '_xxx' (where xxx is the channel number) will be added to it for
+                each output channel. If `name` is a list of strings, They will be
+                used as is and there must be one for each output channel.
+
+        """
+        self._server.setJackOutputPortNames(name)
+
+    def setIsJackTransportSlave(self, x):
+        """
+        Set if pyo's server is slave to jack transport or not.
+
+        This method must be called before booting the server.
+
+        :Args:
+
+            x: boolean
+                If True, the server's start and stop command will be slave to
+                Jack transport. If False (the default) jack transport is ignored.
+
+        """
+        self._isJackTransportSlave = x
+        self._server.setIsJackTransportSlave(x)
+
     def setGlobalSeed(self, x):
         """
         Set the server's global seed used by random objects.
@@ -908,12 +1011,18 @@ class Server(object):
         """
         Add a MIDI event in the server processing loop.
 
-        This method can be used to  programatically simulate incomming
+        This method can be used to  programmatically simulate incoming
         MIDI events. In an embedded framework (ie. pyo inside puredata,
         openframeworks, etc.), this is useful to control a MIDI-driven
         script from the host program. Arguments can be list of values to
         generate multiple events in one call.
 
+        The MIDI event buffer is emptied at the end of each processing
+        block. So, for events to be processed, addMidiEvent should be
+        called at the beginning of the block. If you use audio objects
+        to generate MIDI events, they should be created before the rest
+        of the processing chain.
+
         :Args:
 
             status: int
@@ -964,6 +1073,20 @@ class Server(object):
         """
         return self._server.getBufferSize()
 
+    def getGlobalDur(self):
+        """
+        Return the current global duration.
+
+        """
+        return self._server.getGlobalDur()
+
+    def getGlobalDel(self):
+        """
+        Return the current global delay time.
+
+        """
+        return self._server.getGlobalDel()
+
     def getGlobalSeed(self):
         """
         Return the current global seed.
