@@ -49,6 +49,9 @@ int Server_jack_init(Server *self) { return -10; };
 int Server_jack_deinit(Server *self) { return 0; };
 int Server_jack_start(Server *self) { return 0; };
 int Server_jack_stop(Server *self) { return 0; };
+int jack_input_port_set_names(Server *self) { return 0; };
+int jack_output_port_set_names(Server *self) { return 0; };
+
 #endif
 
 #ifdef USE_COREAUDIO
@@ -73,6 +76,7 @@ void pm_programout(Server *self, int value, int chan, long timestamp) {};
 void pm_pressout(Server *self, int value, int chan, long timestamp) {};
 void pm_bendout(Server *self, int value, int chan, long timestamp) {};
 void pm_sysexout(Server *self, unsigned char *msg, long timestamp) {};
+long pm_get_current_time() { return 0; };
 #endif
 
 /** Array of Server objects. **/
@@ -191,6 +195,7 @@ void
     Server *self;
     self = (Server *)arg;
 
+    PyGILState_STATE s = PyGILState_Ensure();
     if (self->recdur < 0) {
         Server_error(self,"Duration must be specified for Offline Server (see Server.recordOptions).");
     }
@@ -207,6 +212,8 @@ void
         sf_close(self->recfile);
         Server_message(self,"Offline Server rendering finished.\n");
     }
+    PyGILState_Release(s);
+
     return NULL;
 }
 
@@ -350,6 +357,9 @@ Server_process_buffers(Server *server)
     */
     PyGILState_STATE s = PyGILState_Ensure();
 
+    if (server->elapsedSamples == 0)
+        server->midi_time_offset = pm_get_current_time();
+
     if (server->CALLBACK != NULL)
         PyObject_Call((PyObject *)server->CALLBACK, PyTuple_New(0), NULL);
 
@@ -485,60 +495,6 @@ Server_process_time(Server *server)
     }
 }
 
-PyObject *
-Server_shut_down(Server *self)
-{
-    int i;
-    int ret = -1;
-    if (self->server_booted == 0) {
-        Server_error(self, "The Server must be booted!\n");
-        Py_RETURN_NONE;
-    }
-    if (self->server_started == 1) {
-        Server_stop((Server *)self);
-    }
-
-    for (i=0; i<num_rnd_objs; i++) {
-        rnd_objs_count[i] = 0;
-    }
-
-    switch (self->midi_be_type) {
-        case PyoPortmidi:
-            if (self->withPortMidi == 1 || self->withPortMidiOut == 1)
-                ret = Server_pm_deinit(self);
-            break;
-        default:
-            break;
-    }
-
-    switch (self->audio_be_type) {
-        case PyoPortaudio:
-            ret = Server_pa_deinit(self);
-            break;
-        case PyoCoreaudio:
-            ret = Server_coreaudio_deinit(self);
-            break;
-        case PyoJack:
-            ret = Server_jack_deinit(self);
-            break;
-        case PyoOffline:
-            ret = Server_offline_deinit(self);
-            break;
-        case PyoOfflineNB:
-            ret = Server_offline_deinit(self);
-            break;
-        case PyoEmbedded:
-            ret = Server_embedded_deinit(self);
-            break;
-    }
-    self->server_booted = 0;
-    if (ret < 0) {
-        Server_error(self, "Error closing audio backend.\n");
-    }
-
-    Py_RETURN_NONE;
-}
-
 static int
 Server_traverse(Server *self, visitproc visit, void *arg)
 {
@@ -569,7 +525,7 @@ static void
 Server_dealloc(Server* self)
 {
     if (self->server_booted == 1)
-        Server_shut_down(self);
+        Server_shutdown(self);
     Server_clear(self);
     free(self->input_buffer);
     free(self->output_buffer);
@@ -620,6 +576,8 @@ Server_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->jackautoout = 1;
     self->jackAutoConnectInputPorts = PyList_New(0);
     self->jackAutoConnectOutputPorts = PyList_New(0);
+    self->isJackTransportSlave = 0;
+    self->jack_transport_state = 0;
     self->samplingRate = 44100.0;
     self->nchnls = 2;
     self->ichnls = 2;
@@ -637,8 +595,9 @@ Server_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->midi_input = -1;
     self->midi_output = -1;
     self->midiActive = 1;
+    self->midi_time_offset = 0;
     self->amp = self->resetAmp = 1.;
-    self->currentAmp = self->lastAmp = 0.;
+    self->currentAmp = self->lastAmp = 1.; // If set to 0, there is a 5ms fadein at server start.
     self->withGUI = 0;
     self->withTIME = 0;
     self->verbosity = 7;
@@ -646,6 +605,8 @@ Server_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->recformat = 0;
     self->rectype = 0;
     self->recquality = 0.4;
+    self->globalDur = 0.0;
+    self->globalDel = 0.0;
     self->startoffset = 0.0;
     self->globalSeed = 0;
     self->CALLBACK = NULL;
@@ -872,6 +833,24 @@ Server_setBufferSize(Server *self, PyObject *arg)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+Server_setGlobalDur(Server *self, PyObject *arg)
+{
+    if (arg != NULL && PyNumber_Check(arg)) {
+        self->globalDur = PyFloat_AsDouble(arg);
+    }
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+Server_setGlobalDel(Server *self, PyObject *arg)
+{
+    if (arg != NULL && PyNumber_Check(arg)) {
+        self->globalDel = PyFloat_AsDouble(arg);
+    }
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 Server_setDuplex(Server *self, PyObject *arg)
 {
@@ -935,6 +914,58 @@ Server_setJackAutoConnectOutputPorts(Server *self, PyObject *arg)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+Server_setJackInputPortNames(Server *self, PyObject *arg)
+{
+    PyObject *tmp;
+
+    if (arg != NULL) {
+        if (PyList_Check(arg) || PY_STRING_CHECK(arg)) {
+            tmp = arg;
+            Py_XDECREF(self->jackInputPortNames);
+            Py_INCREF(tmp);
+            self->jackInputPortNames = tmp;
+
+            jack_input_port_set_names(self);
+        }
+    }
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+Server_setJackOutputPortNames(Server *self, PyObject *arg)
+{
+    PyObject *tmp;
+
+    if (arg != NULL) {
+        if (PyList_Check(arg) || PY_STRING_CHECK(arg)) {
+            tmp = arg;
+            Py_XDECREF(self->jackOutputPortNames);
+            Py_INCREF(tmp);
+            self->jackOutputPortNames = tmp;
+
+            jack_output_port_set_names(self);
+        }
+    }
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+Server_setIsJackTransportSlave(Server *self, PyObject *arg)
+{
+    if (self->server_booted) {
+        Server_warning(self,"Can't change isJackTransportSlave mode for booted server.\n");
+        Py_RETURN_NONE;
+    }
+    if (arg != NULL) {
+        if (PyInt_Check(arg))
+            self->isJackTransportSlave = PyInt_AsLong(arg);
+    }
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 Server_setGlobalSeed(Server *self, PyObject *arg)
 {
@@ -1082,11 +1113,67 @@ Server_setStartOffset(Server *self, PyObject *arg)
     Py_RETURN_NONE;
 }
 
-static PyObject *
+/*******************************************/
+/** Server shutdown / boot / start / stop **/
+/*******************************************/
+
+PyObject *
+Server_shutdown(Server *self)
+{
+    int i, ret = -1;
+    if (self->server_booted == 0) {
+        Server_error(self, "The Server must be booted!\n");
+        Py_RETURN_NONE;
+    }
+    if (self->server_started == 1) {
+        Server_stop((Server *)self);
+    }
+
+    for (i=0; i<num_rnd_objs; i++) {
+        rnd_objs_count[i] = 0;
+    }
+
+    switch (self->midi_be_type) {
+        case PyoPortmidi:
+            if (self->withPortMidi == 1 || self->withPortMidiOut == 1)
+                ret = Server_pm_deinit(self);
+            break;
+        default:
+            break;
+    }
+
+    switch (self->audio_be_type) {
+        case PyoPortaudio:
+            ret = Server_pa_deinit(self);
+            break;
+        case PyoCoreaudio:
+            ret = Server_coreaudio_deinit(self);
+            break;
+        case PyoJack:
+            ret = Server_jack_deinit(self);
+            break;
+        case PyoOffline:
+            ret = Server_offline_deinit(self);
+            break;
+        case PyoOfflineNB:
+            ret = Server_offline_deinit(self);
+            break;
+        case PyoEmbedded:
+            ret = Server_embedded_deinit(self);
+            break;
+    }
+    self->server_booted = 0;
+    if (ret < 0) {
+        Server_error(self, "Error closing audio backend.\n");
+    }
+
+    Py_RETURN_NONE;
+}
+
+PyObject *
 Server_boot(Server *self, PyObject *arg)
 {
-    int audioerr = 0, midierr = 0;
-    int i;
+    int i, audioerr = 0, midierr = 0;
     if (self->server_booted == 1) {
         Server_error(self, "Server already booted!\n");
         Py_RETURN_NONE;
@@ -1095,6 +1182,11 @@ Server_boot(Server *self, PyObject *arg)
     self->stream_count = 0;
     self->elapsedSamples = 0;
 
+    /* Ensure Python is set up for threading */
+    if (!PyEval_ThreadsInitialized()) {
+        PyEval_InitThreads();
+    }
+
     int needNewBuffer = 0;
     if (arg != NULL && PyBool_Check(arg)) {
         needNewBuffer = PyObject_IsTrue(arg);
@@ -1189,7 +1281,7 @@ Server_boot(Server *self, PyObject *arg)
     Py_RETURN_NONE;
 }
 
-static PyObject *
+PyObject *
 Server_start(Server *self)
 {
     int err = -1;
@@ -1205,14 +1297,9 @@ Server_start(Server *self)
 
     Server_debug(self, "Server_start: number of streams %d\n", self->stream_count);
 
-    /* Ensure Python is set up for threading */
-    if (!PyEval_ThreadsInitialized()) {
-        PyEval_InitThreads();
-    }
-
     self->server_stopped = 0;
     self->server_started = 1;
-    self->timeStep = (int)(0.01 * self->samplingRate);
+    self->timeStep = (int)(0.005 * self->samplingRate);
 
     if (self->startoffset > 0.0) {
         Server_message(self,"Rendering %.2f seconds offline...\n", self->startoffset);
@@ -1251,6 +1338,9 @@ Server_start(Server *self)
         Server_error(self, "Error starting server.\n");
     }
 
+    if (self->withGUI && PyObject_HasAttrString((PyObject *)self->GUI, "setStartButtonState"))
+        PyObject_CallMethod((PyObject *)self->GUI, "setStartButtonState", "i", 1);
+
     Py_RETURN_NONE;
 }
 
@@ -1282,11 +1372,11 @@ Server_stop(Server *self)
     }
     else {
         self->server_stopped = 1;
+        self->server_started = 0;
     }
 
-    /* This call is needed to recover from thread fork with python3/jack on debian.*/
-    /* TODO: Need to be tested with other OSes and audio driver. */
-    PyOS_AfterFork();
+    if (self->withGUI && PyObject_HasAttrString((PyObject *)self->GUI, "setStartButtonState"))
+        PyObject_CallMethod((PyObject *)self->GUI, "setStartButtonState", "i", 0);
 
     Py_RETURN_NONE;
 }
@@ -1673,6 +1763,15 @@ Server_getMidiEventCount(Server *self) {
     return self->midi_count;
 }
 
+long
+Server_getMidiTimeOffset(Server *self) {
+    return self->midi_time_offset;
+}
+
+unsigned long Server_getElapsedTime(Server *self) {
+    return self->elapsedSamples;
+}
+
 static PyObject *
 Server_addMidiEvent(Server *self, PyObject *args)
 {
@@ -1734,6 +1833,18 @@ Server_getBufferSize(Server *self)
         return PyInt_FromLong(self->bufferSize * self->currentResampling);
 }
 
+static PyObject *
+Server_getGlobalDur(Server *self)
+{
+    return PyFloat_FromDouble(self->globalDur);
+}
+
+static PyObject *
+Server_getGlobalDel(Server *self)
+{
+    return PyFloat_FromDouble(self->globalDel);
+}
+
 static PyObject *
 Server_beginResamplingBlock(Server *self, PyObject *arg)
 {
@@ -1894,6 +2005,8 @@ static PyMethodDef Server_methods[] = {
     {"deactivateMidi", (PyCFunction)Server_deactivateMidi, METH_NOARGS, "Deactivates midi callback."},
     {"setSamplingRate", (PyCFunction)Server_setSamplingRate, METH_O, "Sets the server's sampling rate."},
     {"setBufferSize", (PyCFunction)Server_setBufferSize, METH_O, "Sets the server's buffer size."},
+    {"setGlobalDur", (PyCFunction)Server_setGlobalDur, METH_O, "Sets the server's globalDur attribute."},
+    {"setGlobalDel", (PyCFunction)Server_setGlobalDel, METH_O, "Sets the server's globalDel attribute."},
     {"beginResamplingBlock", (PyCFunction)Server_beginResamplingBlock, METH_O, "Starts a resampling code block."},
     {"endResamplingBlock", (PyCFunction)Server_endResamplingBlock, METH_NOARGS, "Stops a resampling code block."},
     {"setNchnls", (PyCFunction)Server_setNchnls, METH_O, "Sets the server's number of output/input channels."},
@@ -1902,6 +2015,9 @@ static PyMethodDef Server_methods[] = {
     {"setJackAuto", (PyCFunction)Server_setJackAuto, METH_VARARGS, "Tells the server to auto-connect Jack ports (0 = disable, 1 = enable)."},
     {"setJackAutoConnectInputPorts", (PyCFunction)Server_setJackAutoConnectInputPorts, METH_O, "Sets a list of ports to auto-connect inputs when using Jack."},
     {"setJackAutoConnectOutputPorts", (PyCFunction)Server_setJackAutoConnectOutputPorts, METH_O, "Sets a list of ports to auto-connect outputs when using Jack."},
+    {"setJackInputPortNames", (PyCFunction)Server_setJackInputPortNames, METH_O, "Sets the short name of input ports for jack server."},
+    {"setJackOutputPortNames", (PyCFunction)Server_setJackOutputPortNames, METH_O, "Sets the short name of output ports for jack server."},
+    {"setIsJackTransportSlave", (PyCFunction)Server_setIsJackTransportSlave, METH_O, "Sets if the server's start/stop is slave of jack transport."},
     {"setGlobalSeed", (PyCFunction)Server_setGlobalSeed, METH_O, "Sets the server's global seed for random objects."},
     {"setAmp", (PyCFunction)Server_setAmp, METH_O, "Sets the overall amplitude."},
     {"setAmpCallable", (PyCFunction)Server_setAmpCallable, METH_O, "Sets the Server's GUI callable object."},
@@ -1910,7 +2026,7 @@ static PyMethodDef Server_methods[] = {
     {"setVerbosity", (PyCFunction)Server_setVerbosity, METH_O, "Sets the verbosity."},
     {"setStartOffset", (PyCFunction)Server_setStartOffset, METH_O, "Sets starting time offset."},
     {"boot", (PyCFunction)Server_boot, METH_O, "Setup and boot the server."},
-    {"shutdown", (PyCFunction)Server_shut_down, METH_NOARGS, "Shut down the server."},
+    {"shutdown", (PyCFunction)Server_shutdown, METH_NOARGS, "Shut down the server."},
     {"start", (PyCFunction)Server_start, METH_NOARGS, "Starts the server's callback loop."},
     {"stop", (PyCFunction)Server_stop, METH_NOARGS, "Stops the server's callback loop."},
     {"recordOptions", (PyCFunction)Server_recordOptions, METH_VARARGS|METH_KEYWORDS, "Sets format settings for offline rendering and global recording."},
@@ -1936,6 +2052,8 @@ static PyMethodDef Server_methods[] = {
     {"getIchnls", (PyCFunction)Server_getIchnls, METH_NOARGS, "Returns the server's current number of input channels."},
     {"getGlobalSeed", (PyCFunction)Server_getGlobalSeed, METH_NOARGS, "Returns the server's global seed."},
     {"getBufferSize", (PyCFunction)Server_getBufferSize, METH_NOARGS, "Returns the server's buffer size."},
+    {"getGlobalDur", (PyCFunction)Server_getGlobalDur, METH_NOARGS, "Returns the server's globalDur attribute."},
+    {"getGlobalDel", (PyCFunction)Server_getGlobalDel, METH_NOARGS, "Returns the server's globalDel attribute."},
     {"getIsBooted", (PyCFunction)Server_getIsBooted, METH_NOARGS, "Returns 1 if the server is booted, otherwise returns 0."},
     {"getIsStarted", (PyCFunction)Server_getIsStarted, METH_NOARGS, "Returns 1 if the server is started, otherwise returns 0."},
     {"getMidiActive", (PyCFunction)Server_getMidiActive, METH_NOARGS, "Returns 1 if midi callback is active, otherwise returns 0."},
