@@ -19,6 +19,7 @@
  *************************************************************************/
 
 #include <Python.h>
+#include "py2to3.h"
 #include "structmember.h"
 #include "pyomodule.h"
 #include "streammodule.h"
@@ -111,7 +112,7 @@ Record_dealloc(Record* self)
     pyo_DEALLOC
     free(self->buffer);
     Record_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -120,6 +121,7 @@ Record_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     int i, buflen;
     int fileformat = 0;
     int sampletype = 0;
+    double quality = 0.4;
     PyObject *input_listtmp;
     Record *self;
     self = (Record *)type->tp_alloc(type, 0);
@@ -132,9 +134,9 @@ Record_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     Stream_setFunctionPtr(self->stream, Record_compute_next_data_frame);
     self->mode_func_ptr = Record_setProcMode;
 
-    static char *kwlist[] = {"input", "filename", "chnls", "fileformat", "sampletype", "buffering", NULL};
+    static char *kwlist[] = {"input", "filename", "chnls", "fileformat", "sampletype", "buffering", "quality", NULL};
 
-    if (! PyArg_ParseTupleAndKeywords(args, kwds, "Os|iiii", kwlist, &input_listtmp, &self->recpath, &self->chnls, &fileformat, &sampletype, &self->buffering))
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "Os|iiiid", kwlist, &input_listtmp, &self->recpath, &self->chnls, &fileformat, &sampletype, &self->buffering, &quality))
         Py_RETURN_NONE;
 
     Py_XDECREF(self->input_list);
@@ -203,10 +205,15 @@ Record_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
     /* Open the output file. */
     if (! (self->recfile = sf_open(self->recpath, SFM_WRITE, &self->recinfo))) {
-        printf ("Not able to open output file %s.\n", self->recpath);
+        PySys_WriteStdout("Record: not able to open output file %s.\n", self->recpath);
         Py_RETURN_NONE;
     }
 
+    // Sets the encoding quality for FLAC and OGG compressed formats
+    if (fileformat == 5 || fileformat == 7) {
+        sf_command(self->recfile, SFC_SET_VBR_ENCODING_QUALITY, &quality, sizeof(double));
+    }
+
     buflen = self->bufsize * self->chnls * self->buffering;
     self->buffer = (MYFLT *)realloc(self->buffer, buflen * sizeof(MYFLT));
     for (i=0; i<buflen; i++) {
@@ -246,8 +253,7 @@ static PyMethodDef Record_methods[] = {
 };
 
 PyTypeObject RecordType = {
-PyObject_HEAD_INIT(NULL)
-0,                                              /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Record_base",                                   /*tp_name*/
 sizeof(Record),                                 /*tp_basicsize*/
 0,                                              /*tp_itemsize*/
@@ -255,7 +261,7 @@ sizeof(Record),                                 /*tp_basicsize*/
 0,                                              /*tp_print*/
 0,                                              /*tp_getattr*/
 0,                                              /*tp_setattr*/
-0,                                              /*tp_compare*/
+0,                                              /*tp_as_async (tp_compare in Python 2)*/
 0,                                              /*tp_repr*/
 0,												/*tp_as_number*/
 0,                                              /*tp_as_sequence*/
@@ -370,7 +376,7 @@ ControlRec_dealloc(ControlRec* self)
     if (self->buffer != NULL)
         free(self->buffer);
     ControlRec_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -470,8 +476,7 @@ static PyMethodDef ControlRec_methods[] = {
 };
 
 PyTypeObject ControlRecType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                                              /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.ControlRec_base",                                   /*tp_name*/
     sizeof(ControlRec),                                 /*tp_basicsize*/
     0,                                              /*tp_itemsize*/
@@ -479,7 +484,7 @@ PyTypeObject ControlRecType = {
     0,                                              /*tp_print*/
     0,                                              /*tp_getattr*/
     0,                                              /*tp_setattr*/
-    0,                                              /*tp_compare*/
+    0,                                              /*tp_as_async (tp_compare in Python 2)*/
     0,                                              /*tp_repr*/
     0,												/*tp_as_number*/
     0,                                              /*tp_as_sequence*/
@@ -645,7 +650,7 @@ ControlRead_dealloc(ControlRead* self)
     free(self->values);
     free(self->trigsBuffer);
     ControlRead_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -739,15 +744,12 @@ ControlRead_setValues(ControlRead *self, PyObject *arg)
 {
     Py_ssize_t i;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->size = PyList_Size(arg);
     self->values = (MYFLT *)realloc(self->values, self->size * sizeof(MYFLT));
     for (i=0; i<self->size; i++) {
-        self->values[i] = PyFloat_AS_DOUBLE(PyList_GET_ITEM(arg, i));
+        self->values[i] = PyFloat_AsDouble(PyList_GET_ITEM(arg, i));
     }
 
 	Py_INCREF(Py_None);
@@ -757,10 +759,7 @@ ControlRead_setValues(ControlRead *self, PyObject *arg)
 static PyObject *
 ControlRead_setRate(ControlRead *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->rate = PyInt_AsLong(arg);
     self->modulo = (int)(self->sr / self->rate);
@@ -772,10 +771,7 @@ ControlRead_setRate(ControlRead *self, PyObject *arg)
 static PyObject *
 ControlRead_setLoop(ControlRead *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->loop = PyInt_AsLong(arg);
 
@@ -786,10 +782,7 @@ ControlRead_setLoop(ControlRead *self, PyObject *arg)
 static PyObject *
 ControlRead_setInterp(ControlRead *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     int isNumber = PyNumber_Check(arg);
 
@@ -833,7 +826,7 @@ static PyNumberMethods ControlRead_as_number = {
     (binaryfunc)ControlRead_add,                      /*nb_add*/
     (binaryfunc)ControlRead_sub,                 /*nb_subtract*/
     (binaryfunc)ControlRead_multiply,                 /*nb_multiply*/
-    (binaryfunc)ControlRead_div,                   /*nb_divide*/
+    INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
     0,                /*nb_remainder*/
     0,                   /*nb_divmod*/
     0,                   /*nb_power*/
@@ -847,16 +840,16 @@ static PyNumberMethods ControlRead_as_number = {
     0,              /*nb_and*/
     0,              /*nb_xor*/
     0,               /*nb_or*/
-    0,                                          /*nb_coerce*/
+    INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
     0,                       /*nb_int*/
     0,                      /*nb_long*/
     0,                     /*nb_float*/
-    0,                       /*nb_oct*/
-    0,                       /*nb_hex*/
+    INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+    INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
     (binaryfunc)ControlRead_inplace_add,              /*inplace_add*/
     (binaryfunc)ControlRead_inplace_sub,         /*inplace_subtract*/
     (binaryfunc)ControlRead_inplace_multiply,         /*inplace_multiply*/
-    (binaryfunc)ControlRead_inplace_div,           /*inplace_divide*/
+    INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
     0,        /*inplace_remainder*/
     0,           /*inplace_power*/
     0,       /*inplace_lshift*/
@@ -865,15 +858,14 @@ static PyNumberMethods ControlRead_as_number = {
     0,      /*inplace_xor*/
     0,       /*inplace_or*/
     0,             /*nb_floor_divide*/
-    0,              /*nb_true_divide*/
+    (binaryfunc)ControlRead_div,                       /*nb_true_divide*/
     0,     /*nb_inplace_floor_divide*/
-    0,      /*nb_inplace_true_divide*/
+    (binaryfunc)ControlRead_inplace_div,                       /*nb_inplace_true_divide*/
     0,                     /* nb_index */
 };
 
 PyTypeObject ControlReadType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.ControlRead_base",         /*tp_name*/
     sizeof(ControlRead),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -881,7 +873,7 @@ PyTypeObject ControlReadType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     &ControlRead_as_number,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
@@ -997,7 +989,7 @@ NoteinRec_dealloc(NoteinRec* self)
 {
     pyo_DEALLOC
     NoteinRec_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -1089,8 +1081,7 @@ static PyMethodDef NoteinRec_methods[] = {
 };
 
 PyTypeObject NoteinRecType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                                              /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.NoteinRec_base",                                   /*tp_name*/
     sizeof(NoteinRec),                                 /*tp_basicsize*/
     0,                                              /*tp_itemsize*/
@@ -1098,7 +1089,7 @@ PyTypeObject NoteinRecType = {
     0,                                              /*tp_print*/
     0,                                              /*tp_getattr*/
     0,                                              /*tp_setattr*/
-    0,                                              /*tp_compare*/
+    0,                                              /*tp_as_async (tp_compare in Python 2)*/
     0,                                              /*tp_repr*/
     0,												/*tp_as_number*/
     0,                                              /*tp_as_sequence*/
@@ -1260,7 +1251,7 @@ NoteinRead_dealloc(NoteinRead* self)
     free(self->timestamps);
     free(self->trigsBuffer);
     NoteinRead_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -1353,15 +1344,12 @@ NoteinRead_setValues(NoteinRead *self, PyObject *arg)
 {
     Py_ssize_t i;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->size = PyList_Size(arg);
     self->values = (MYFLT *)realloc(self->values, self->size * sizeof(MYFLT));
     for (i=0; i<self->size; i++) {
-        self->values[i] = PyFloat_AS_DOUBLE(PyList_GET_ITEM(arg, i));
+        self->values[i] = PyFloat_AsDouble(PyList_GET_ITEM(arg, i));
     }
 
 	Py_INCREF(Py_None);
@@ -1373,15 +1361,12 @@ NoteinRead_setTimestamps(NoteinRead *self, PyObject *arg)
 {
     Py_ssize_t i;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->size = PyList_Size(arg);
     self->timestamps = (long *)realloc(self->timestamps, self->size * sizeof(long));
     for (i=0; i<self->size; i++) {
-        self->timestamps[i] = (long)(PyFloat_AS_DOUBLE(PyList_GET_ITEM(arg, i)) * self->sr);
+        self->timestamps[i] = (long)(PyFloat_AsDouble(PyList_GET_ITEM(arg, i)) * self->sr);
     }
 
 	Py_INCREF(Py_None);
@@ -1391,10 +1376,7 @@ NoteinRead_setTimestamps(NoteinRead *self, PyObject *arg)
 static PyObject *
 NoteinRead_setLoop(NoteinRead *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->loop = PyInt_AsLong(arg);
 
@@ -1431,7 +1413,7 @@ static PyNumberMethods NoteinRead_as_number = {
     (binaryfunc)NoteinRead_add,                      /*nb_add*/
     (binaryfunc)NoteinRead_sub,                 /*nb_subtract*/
     (binaryfunc)NoteinRead_multiply,                 /*nb_multiply*/
-    (binaryfunc)NoteinRead_div,                   /*nb_divide*/
+    INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
     0,                /*nb_remainder*/
     0,                   /*nb_divmod*/
     0,                   /*nb_power*/
@@ -1445,16 +1427,16 @@ static PyNumberMethods NoteinRead_as_number = {
     0,              /*nb_and*/
     0,              /*nb_xor*/
     0,               /*nb_or*/
-    0,                                          /*nb_coerce*/
+    INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
     0,                       /*nb_int*/
     0,                      /*nb_long*/
     0,                     /*nb_float*/
-    0,                       /*nb_oct*/
-    0,                       /*nb_hex*/
+    INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+    INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
     (binaryfunc)NoteinRead_inplace_add,              /*inplace_add*/
     (binaryfunc)NoteinRead_inplace_sub,         /*inplace_subtract*/
     (binaryfunc)NoteinRead_inplace_multiply,         /*inplace_multiply*/
-    (binaryfunc)NoteinRead_inplace_div,           /*inplace_divide*/
+    INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
     0,        /*inplace_remainder*/
     0,           /*inplace_power*/
     0,       /*inplace_lshift*/
@@ -1463,15 +1445,14 @@ static PyNumberMethods NoteinRead_as_number = {
     0,      /*inplace_xor*/
     0,       /*inplace_or*/
     0,             /*nb_floor_divide*/
-    0,              /*nb_true_divide*/
+    (binaryfunc)NoteinRead_div,                       /*nb_true_divide*/
     0,     /*nb_inplace_floor_divide*/
-    0,      /*nb_inplace_true_divide*/
+    (binaryfunc)NoteinRead_inplace_div,                       /*nb_inplace_true_divide*/
     0,                     /* nb_index */
 };
 
 PyTypeObject NoteinReadType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.NoteinRead_base",         /*tp_name*/
     sizeof(NoteinRead),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -1479,7 +1460,7 @@ PyTypeObject NoteinReadType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     &NoteinRead_as_number,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
